from enum import Enum


class EnumG(Enum):

    Msg_Header = 0x5A
    # Msg_Header = 0xC5
    # 消息类别
    Msg_Type_Bit_App = 1
    Msg_Type_Bit_Base = 2
    Msg_Type_Bit_Update = 4
    Msg_Type_Bit_Test = 5

    # base mid
    BaseMid_GetCapabilities = 0x00
    BaseMid_SetPower = 0x01
    BaseMid_GetPower = 0x02
    BaseMid_SetFreqRange = 0x03
    BaseMid_GetFreqRange = 0x04
    BaseMid_SetFrequency = 0x05
    BaseMid_GetFrequency = 0x06
    BaseMid_SetAntennaHub = 0x07
    BaseMid_GetAntennaHub = 0x08
    BaseMid_SetTagLog = 0x09
    BaseMid_GetTagLog = 0x0A
    BaseMid_SetBaseband = 0x0B
    BaseMid_GetBaseband = 0x0C
    BaseMid_SetAutoDormancy = 0x0D
    BaseMid_GetAutoDormancy = 0x0E
    BaseMid_SetResidenceTime = 0xE0
    BaseMid_GetResidenceTime = 0xE1
    BaseMid_SetGbBaseBand = 0xE2
    BaseMid_GetGbBaseBand = 0xE3
    BaseMid_InventoryEpc = 0x10
    BaseMid_WriteEpc = 0x11
    BaseMid_LockEpc = 0x12
    BaseMid_DestroyEpc = 0x13
    BaseMid_MonzaQT = 0x14
    BaseMid_SuperRW = 0x1A
    BaseMid_Inventory6b = 0x40
    BaseMid_Write6b = 0x41
    BaseMid_Lock6b = 0x42
    BaseMid_Lock6bGet = 0x43
    BaseMid_InventoryGb = 0x50
    BaseMid_WriteGb = 0x51
    BaseMid_LockGb = 0x52
    BaseMid_DestroyGb = 0x53
    BaseMid_InventoryGJb = 0x60
    BaseMid_WriteGJb = 0x61
    BaseMid_LockGJb = 0x62
    BaseMid_DestroyGJb = 0x63
    BaseMid_SafeAttestation = 0xF0
    BaseMid_Hybrid = 0xA0
    BaseMid_Stop = 0xFF

    # app mid
    AppMid_GetReaderInfo = 0x00
    AppMid_GetBaseVersion = 0x01
    AppMid_SetSerialParam = 0x02
    AppMid_GetSerialParam = 0x03
    AppMid_SetReaderIP = 0x04
    AppMid_GetReaderIP = 0x05
    AppMid_GetReaderMac = 0x06
    AppMid_SetTcpMode = 0x07
    AppMid_GetTcpMode = 0x08
    AppMid_SetGpo = 0x09
    AppMid_GetGpiState = 0x0A
    AppMid_SetGpiTrigger = 0x0B
    AppMid_GetGpiTrigger = 0x0C
    AppMid_SetWigan = 0x0D
    AppMid_GetWigan = 0x0E
    AppMid_Reset = 0x0F
    AppMid_SetReaderTime = 0x10
    AppMid_GetReaderTime = 0x11
    AppMid_Heartbeat = 0x12
    AppMid_SetReaderMac = 0x13
    AppMid_RestoreDefault = 0x14
    AppMid_SetRs485 = 0x15
    AppMid_GetRs485 = 0x16
    AppMid_SetBreakpointResume = 0x17
    AppMid_GetBreakpointResume = 0x18
    AppMid_GetCacheTagData = 0x1B
    AppMid_ClearCacheTagData = 0x1C
    AppMid_TagDataReply = 0x1D
    AppMid_SetBeepOnOff = 0x1E
    AppMid_SetBeep = 0x1F
    AppMid_GetWhiteList = 0x20
    AppMid_ImportWhiteList = 0x21
    AppMid_DelWhiteList = 0x22
    AppMid_SetWhiteListActionParam = 0x23
    AppMid_GetWhiteListActionParam = 0x24
    AppMid_SetWhiteListSwitch = 0x25
    AppMid_GetWhiteListSwitch = 0x26
    AppMid_SetUdpParam = 0x27
    AppMid_GetUdpParam = 0x28
    AppMid_SetHttpParam = 0x29
    AppMid_GetHttpParam = 0x2A
    AppMid_SetWifiHotspotSearch = 0x31
    AppMid_GetWifiHotspotSearch = 0x32
    AppMid_SetWifiHotspot = 0x33
    AppMid_GetWifiConnectStatus = 0x34
    AppMid_SetWifiIp = 0x35
    AppMid_GetWifiIp = 0x36
    AppMid_SetWifiOnOff = 0x37
    AppMid_GetWifiOnOff = 0x38

    # EAS定制
    AppMid_SetEasAlarm = 0x3F
    AppMid_GetEasAlarm = 0x40
    AppMid_SetAlarmPerform = 0x23
    AppMid_GetAlarmPerform = 0x24

    # 基带主动上传指令
    BaseLogMid_Epc = 0x00
    BaseLogMid_EpcOver = 0x01
    BaseLogMid_EpcAntId = 0x02
    BaseLogMid_6b = 0x20
    BaseLogMid_6bOver = 0x21
    BaseLogMid_Gb = 0x30
    BaseLogMid_GbOver = 0x31
    BaseLogMid_GJb = 0x40
    BaseLogMid_GJbOver = 0x41

    # 应用主动上传指令 mid
    AppLogMid_gpi = 0x00
    AppLogMid_gpiOver = 0x01
    AppLogMid_allGpiState = 0x02

    # 升级指令 mid
    UpdateMid_App = 0x00
    UpdateMid_Baseband = 0x01

    # 测试指令 mid
    TestMid_CarrierWave = 0x00
    TestMid_DCbias = 0x01
    TestMid_DCbiasGet = 0x02
    TestMid_PowerCalibration = 0x03
    TestMid_PowerCalibrationGet = 0x04
    TestMid_VSWRcheck = 0x05
    TestMid_DCAutobias = 0x06
    TestMid_EnvRssiDetection = 0x07
    TestMid_RssiCalibrationSet = 0x08
    TestMid_RssiCalibrationGet = 0x09
    TestMid_EnvRssiAutoDetection = 0x0A
    TestMid_SerialNoSet = 0x10
    TestMid_ReaderWorkModeSet = 0x11
    TestMid_ReaderWorkModeGet = 0x12
    TestMid_EpcEncryptionSet = 0x13
    TestMid_EpcEncryptionGet = 0x14
    TestMid_R2000ReadWrite = 0x15
    TestMid_R2000ReceivingGain = 0x16

    TestMid_Chip = 0x31

    # 连续 / 单次
    InventoryMode_Single = 0
    InventoryMode_Inventory = 1

    # 锁定类型
    LockMode_Unlock = 0
    LockMode_Lock = 1
    LockMode_PermanentUnlock = 2
    LockMode_PermanentLock = 3

    # 6b标签读取类型
    ReadMode6b_Tid = 0
    ReadMode6b_TidAndUserData = 1
    ReadMode6b_UserData = 2

    # 选择参数匹配区
    ParamFilterArea_EPC = 1
    ParamFilterArea_TID = 2
    ParamFilterArea_UserData = 3

    # 写入标签数据区
    WriteArea_Reserved = 0
    WriteArea_Epc = 1
    WriteArea_Tid = 2
    WriteArea_UserData = 3

    # 6C标签TID读取模式
    ParamTidMode_Auto = 0
    ParamTidMode_Fixed = 1

    # 锁定标签区
    LockArea_Destroy = 0
    LockArea_Access = 1
    LockArea_Epc = 2
    LockArea_Tid = 3
    LockArea_UserData = 4

    AntennaNo_1 = 1
    AntennaNo_2 = 2
    AntennaNo_3 = 4
    AntennaNo_4 = 8
    AntennaNo_5 = 16
    AntennaNo_6 = 32
    AntennaNo_7 = 64
    AntennaNo_8 = 128
    AntennaNo_9 = 256
    AntennaNo_10 = 512
    AntennaNo_11 = 1024
    AntennaNo_12 = 2048
    AntennaNo_13 = 4096
    AntennaNo_14 = 8192
    AntennaNo_15 = 16384
    AntennaNo_16 = 32768
    AntennaNo_17 = 65536
    AntennaNo_18 = 131072
    AntennaNo_19 = 262144
    AntennaNo_20 = 524288
    AntennaNo_21 = 1048576
    AntennaNo_22 = 2097152
    AntennaNo_23 = 4194304
    AntennaNo_24 = 8388608
    AntennaNo_25 = 16777216
    AntennaNo_26 = 33554432
    AntennaNo_27 = 67108864
    AntennaNo_28 = 134217728
    AntennaNo_29 = 268435456
    AntennaNo_30 = 536870912
    AntennaNo_31 = 1073741824
    AntennaNo_32 = 2147483648


from uhf.reader.protocol import *
from uhf.reader.utils import *


class ParamEpcFilter(Parameter):
    # bitLength: int,
    def __init__(self, area: int, bitStart: int, hexData: str):
        self.area = area
        self.bitStart = bitStart
        self.hexData = hexData
        self.bitLength = len(hexData) * 4

    def bytesToClass(self, dataBytes):
        hex = bytesToHex(dataBytes)
        buffer = DynamicBuffer("0x" + hex)
        self.area = buffer.readInt()
        self.bitStart = buffer.readShort()
        self.bitLength = buffer.readInt()
        self.hexData = bytesToHex(buffer.readBytes(self.bitLength))
        return self

    def toBytes(self):
        buffer = DynamicBuffer()
        buffer.putInt(self.area)
        buffer.putShort(self.bitStart)
        buffer.putInt(self.bitLength)
        buffer.putBytes(hexToBytes(self.hexData))
        return buffer.tobytes()

    def __str__(self) -> str:
        return str((self.area, self.bitStart, self.bitLength, self.hexData))



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppSetWifiHotspot(Message):

    def __init__(self, hotspotName: str, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_SetWifiHotspot.value
        self.hotspotName = hotspotName
        self.password = kwargs.get("password", None)
        self.certificationType = kwargs.get("certificationType", 1)
        self.encryptionAlgorithm = kwargs.get("encryptionAlgorithm", 0)

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        if self.hotspotName:
            buffer.putShort(len(self.hotspotName))
            buffer.putBytes(self.hotspotName.encode())
        if self.password is not None:
            buffer.putInt(0x01)
            buffer.putShort(len(self.password))
            buffer.putBytes(self.password.encode())

        if self.certificationType is not None:
            buffer.putInt(0x02)
            buffer.putInt(self.certificationType)

        if self.encryptionAlgorithm is not None:
            buffer.putInt(0x03)
            buffer.putInt(self.encryptionAlgorithm)

        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Set Fail."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



import setuptools

# with open("README.md", "r") as fh:
#     long_description = fh.read()

setuptools.setup(
    # name="uhfReaderApi",
    # version="1.0.4",
    # name="uhfReaderApi_Cykeo",
    # version="1.0.3",
    name="uhfReaderApi_NoHid",
    version="1.0.4",
    author="rfid",
    description="ReaderApi",
    platforms=['linux/Windows'],
    # uhf.
    packages=setuptools.find_namespace_packages(
        exclude=["*.qt", "qt.*", "qt", "*.qt.*", "*.test", "test.*", "test",
                 "*.test.*"]),
    # package_dir={"": "uhf"},
    include_package_data=False,
    # 排除所有 README.md
    # exclude_package_data={'': ['README.md', "setup.py"]},
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'bitstring==3.1.7',
        'pyserial==3.4',
        # 'pyusb==1.1.0',
        # 'hidapi==0.13.1'
    ]
)



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppSetUdpParam(Message):

    def __init__(self, onOrOff: int, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_SetUdpParam.value
        self.onOrOff = onOrOff
        self.ip = kwargs.get("ip", None)
        self.port = kwargs.get("port", None)
        self.period = kwargs.get("period", None)

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putInt(self.onOrOff)
        if self.onOrOff == 1:
            if self.ip is not None:
                buffer.putInt(0x01)
                splitIp = self.ip.split('.')
                for ips in splitIp:
                    buffer.putInt(int(ips))

            if self.port is not None:
                buffer.putInt(0x02)
                buffer.putShort(self.port)

            if self.period is not None:
                buffer.putInt(0x03)
                buffer.putShort(self.period)

        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Fail"}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



from uhf.reader.protocol import *
from uhf.reader.utils import *


class ActionParamFail(Parameter):

    def __init__(self, keepTime: int, **kwargs):
        self.keepTime = keepTime
        self.gpo1 = kwargs.get("gpo1", None)
        self.gpo2 = kwargs.get("gpo2", None)
        self.gpo3 = kwargs.get("gpo3", None)
        self.gpo4 = kwargs.get("gpo4", None)

    def bytesToClass(self, dataBytes):
        hex = bytesToHex(dataBytes)
        buffer = DynamicBuffer("0x" + hex)
        self.keepTime = buffer.readShort()
        while buffer.pos / 8 < len(dataBytes):
            pid = buffer.readInt()
            if pid == 1:
                self.gpo1 = buffer.readInt()
            if pid == 2:
                self.gpo2 = buffer.readInt()
            if pid == 3:
                self.gpo3 = buffer.readInt()
            if pid == 4:
                self.gpo4 = buffer.readInt()
        return self

    def toBytes(self):
        buffer = DynamicBuffer()
        buffer.putShort(self.keepTime)
        if self.gpo1 is not None:
            buffer.putInt(0x01)
            buffer.putInt(self.gpo1)
        if self.gpo2 is not None:
            buffer.putInt(0x02)
            buffer.putInt(self.gpo2)
        if self.gpo3 is not None:
            buffer.putInt(0x03)
            buffer.putInt(self.gpo3)
        if self.gpo4 is not None:
            buffer.putInt(0x04)
            buffer.putInt(self.gpo4)

        return buffer.tobytes()

    def __str__(self) -> str:
        return str((self.keepTime, self.gpo1, self.gpo2, self.gpo3, self.gpo4))



from uhf.reader.communication import CommunicationInter
from uhf.reader.protocol import *
import serial
import threading


class SerialClient(CommunicationInter):

    def __init__(self):
        super().__init__()
        self.client = None
        self.lock = threading.Condition()

    def open(self, param, **kwargs):
        try:
            if not param:
                return False
            # 超时设置,None：永远等待操作，0为立即返回请求结果，其他值为等待超时时间(单位为秒）
            self.client = serial.Serial(param[0], param[1], timeout=kwargs.get("timeout", 0.01))
            if self.client.is_open:
                self.keepReceived = True
                received = threading.Thread(target=self.startReceived, name="SerialReceived")
                received.start()
                processRing = threading.Thread(target=self.startProcess, name="RingBuffer")
                processRing.start()
                return True
        except Exception as ex:
            print(ex.args)
            # raise ex

    def sendMsg(self, message: Message):
        try:
            message.pack()
            if self.isRs485:
                message.mt_13 = "1"
                message.rs485Address = self.rs485Address
            self.client.write(message.toByte(self.isRs485))
            # with self.lock:
            #     print("开始等待")
            #     self.lock.wait(timeout)
        except Exception as ex:
            print(ex.args)

    def sendBytes(self, bytes):
        try:
            self.client.write(bytes)
        except Exception as ex:
            print(ex.args)

    def close(self):
        self.keepReceived = False
        # if self.client:
        #     if self.client.is_open:
        #         self.client.close()
        with self.ringLock:
            self.ringLock.notifyAll()

    def startReceived(self):
        while self.keepReceived:
            try:
                # print('thread %s is running...' % threading.current_thread().name)
                with self.ringLock:
                    recv = self.client.read(1024)  # type:bytes
                    if recv:
                        # print("接收===", recv.hex().upper())
                        self.ringBuffer.writeData(recv)
                        # print("环形内容===", self.ringBuffer.buffer.hex)
                        self.ringLock.notify()
                        self.ringLock.wait()
            except Exception as ex:
                print("startReceived->",ex.args)
                # raise ex
        else:
            if self.client:
                if self.client.is_open:
                    self.client.close()



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppGetReaderInfo(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_GetReaderInfo.value
        self.readerSerialNumber = ""
        self.powerOnTime = ""
        self.baseCompileTime = ""
        self.appVersions = ""
        self.systemVersions = ""
        self.appCompileTime = ""

    def bytesToClass(self):
        pass

    def pack(self):
        super().pack()

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            snLen = buffer.readShort()
            sn = buffer.readBytes(snLen * 8)
            if sn:
                self.readerSerialNumber = listToAscii(sn)
            self.powerOnTime = secondToDhms(buffer.readLong())
            btLen = buffer.readShort()
            bt = buffer.readBytes(btLen * 8)
            if bt:
                self.baseCompileTime = listToAscii(bt)
            while buffer.pos / 8 < len(self.cData):
                pid = buffer.readInt()
                if pid == 1:
                    self.appVersions = str(buffer.readInt()) + "." + str(buffer.readInt()) + "." + str(
                        buffer.readInt()) + "." + str(buffer.readInt())
                elif pid == 2:
                    svLen = buffer.readShort()
                    sv = buffer.readBytes(svLen * 8)
                    if sv:
                        self.systemVersions = listToAscii(sv)
                elif pid == 3:
                    atLen = buffer.readShort()
                    at = buffer.readBytes(atLen * 8)
                    if at:
                        self.appCompileTime = listToAscii(at)
            self.rtCode = 0

    def __str__(self) -> str:
        return str((self.readerSerialNumber, self.powerOnTime, self.baseCompileTime.strip(b'\x00'.decode()),
                    self.appVersions, self.systemVersions, self.appCompileTime))
        # return "{0},{1},{2},{3},{4},{5}".format(self.readerSerialNumber, self.powerOnTime, self.baseCompileTime,
        #                                     self.appVersions, self.systemVersions, self.appCompileTime)



from uhf.reader.communication import CommunicationInter
from uhf.reader.protocol import *
import socket
import threading


class TcpClient(CommunicationInter):

    def __init__(self):
        super().__init__()
        self.client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.lock = threading.Condition()
        self.addr = None
        self.callDisconnected = None
        self.timeout_count = 0

    def open(self, param, **kwargs):
        try:
            if not param:
                return False
            self.client.settimeout(kwargs.get("timeout", 3))
            self.client.connect(param)
            self.client.settimeout(None)
            self.keepReceived = True
            self.addr = param
            received = threading.Thread(target=self.startReceived, name="TcpReceived")
            received.start()
            processRing = threading.Thread(target=self.startProcess, name="RingBuffer")
            processRing.start()
            return True
        except Exception as ex:
            print(ex.args)

    def openSocket(self, serverSocket):
        if serverSocket:
            self.keepReceived = True
            self.client = serverSocket
            received = threading.Thread(target=self.startReceived, name="TcpReceived")
            received.start()
            processRing = threading.Thread(target=self.startProcess, name="RingBuffer")
            processRing.start()
            return True
        return False

    def sendMsg(self, message: Message):
        try:
            message.pack()
            self.client.send(message.toByte(False))
        except Exception as ex:
            print(ex.args)
        # with self.lock:
        #     print("开始等待")
        #     self.lock.wait(timeout)

    def sendBytes(self, bytes):
        self.client.send(bytes)

    def close(self):
        self.keepReceived = False
        if self.client:
            self.client.close()
        with self.ringLock:
            self.ringLock.notifyAll()

    def startReceived(self):
        while self.keepReceived:
            try:
                # print('thread %s is running...' % threading.current_thread().name)
                with self.ringLock:
                    self.client.settimeout(5)
                    recv = self.client.recv(1024)
                    self.timeout_count = 0
                    if recv:
                        # print("接收===", recv.hex().upper())
                        self.ringBuffer.writeData(recv)
                        self.ringLock.notify()
                        self.ringLock.wait()
                    else:
                        self.sendMsg(MsgAppGetBaseVersion())
            except Exception as ex:
                if ex.args[0] == "timed out":
                    self.timeout_count += 1
                    self.sendMsg(MsgAppGetBaseVersion())
                    if self.timeout_count == 3:
                        self.keepReceived = False
                        self.callDisConnect(self.addr)
                else:
                    print(ex.args)
                    # raise ex
                    self.keepReceived = False
                    self.callDisConnect(self.addr)



# import usb.core
# import usb.util


# import time


# import platform
#
# # find our device
# dev = usb.core.find(idVendor=0x03EB, idProduct=0x2421)  # 设备管理器中确认这两个参数值
#
# # was it found?
# if dev is None:
#     raise ValueError('Device not found')
#
# if platform.system().lower() == 'linux':
#     if dev.is_kernel_driver_active(0):
#         dev.detach_kernel_driver(0)  # ＃分离内核驱动程序并通过libusb声明
#
# # set the active configuration. With no arguments, the first
# # configuration will be the active one
# dev.set_configuration()
#
# # get an endpoint instance
# configuration = dev.get_active_configuration()
# # configuration = dev[0]
#
# cfg_ = configuration[(0, 0)]
# # alt = usb.util.find_descriptor(cfg_, find_all=True, bInterfaceNumber=1)
# # print(alt)
#
# # 0x2 写入节点
# outPoint = usb.util.find_descriptor(
#     cfg_,
#     # match the first OUT endpoint
#     custom_match=lambda e: usb.util.endpoint_direction(e.bEndpointAddress) == usb.util.ENDPOINT_OUT)
#
# # 0x81 读取节点
# inPoint = usb.util.find_descriptor(
#     cfg_,
#     # match the first IN endpoint
#     custom_match=lambda e: usb.util.endpoint_direction(e.bEndpointAddress) == usb.util.ENDPOINT_IN)
#
# # assert outPoint, inPoint is not None
#
# msg = [90, 0, 1, 2, 255, 0, 0, 136, 90]  # bytes
# temp0 = [0 for i in range(64 - len(msg))]  # 缺少的字节
#
# msg.extend(temp0)  # 合并后64个字节[1, 2, 3, 4, 5, 6,0,0,0,0,......]
#
# outPoint.write(msg)  # 写入 注意必须满足64个字节
# # dev.write(0x2, msg, 0)  # 效果同上写入
#
# read = inPoint.read(64)  # 读取64个字节
# print(read)
# # read = dev.read(0x81, 64, 0)  # 效果同上读取

# def getUsbHidDeviceList():
#     """
#     获取usb-hid列表
#     :return: usb-hid list
#     """
#     dev = usb.core.find(find_all=True, idVendor=0x03EB, idProduct=0x2421)
#     return [item for item in dev]


# def getUsbHidDeviceDict():
#     """
#     获取usb-hid字典
#     :return: usb-hid dict
#     """
#     dev = usb.core.find(find_all=True, idVendor=0x03EB, idProduct=0x2421)
#     hidDic = {}
#     for item in dev:
#         hidDic[str(item.address)] = item
#     return hidDic

# start = time.time()
# hid_list = getUsbHidDict()
# print(hid_list)
# print(time.time() - start)

# python -c "import usb; print(f'Using PyUSB {usb.__version__}')"
# python -m timeit -s "import usb; list(usb.core.find(find_all=True))"



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgBaseWriteEpc(Message):

    def __init__(self, antennaEnable: int, area: int, start: int, hexWriteData: str, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseMid_WriteEpc.value
        self.antennaEnable = antennaEnable
        self.area = area
        self.start = start
        self.hexWriteData = hexWriteData
        self.filter = kwargs.get("filter", None)  # type:ParamEpcFilter
        self.hexPassword = kwargs.get("hexPassword", None)
        self.block = kwargs.get("block", None)
        self.errorIndex = None

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putLong(self.antennaEnable)
        buffer.putInt(self.area)
        buffer.putShort(self.start)
        if self.hexWriteData:
            to_bytes = hexToBytes(self.hexWriteData)
            buffer.putShort(len(to_bytes))
            buffer.putBytes(to_bytes)
        if self.filter is not None:
            buffer.putInt(0x01)
            filter_bytes = self.filter.toBytes()
            buffer.putShort(len(filter_bytes))
            buffer.putBytes(filter_bytes)
        if self.hexPassword is not None:
            buffer.putInt(0x02)
            buffer.putBytes(hexToBytes(self.hexPassword))
        if self.block is not None:
            buffer.putInt(0x03)
            buffer.putInt(self.block)

        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Port parameter error.",
                      2: "Filter parameter error.", 3: "Write parameter error.", 4: "CRC check error.",
                      5: "Underpower error.", 6: "Data area overflow.", 7: "Data area is locked.",
                      8: "Access password error.", 9: "Other error.", 10: "Label is missing.", 11: "Command error."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)
            if len(self.cData) > 1:
                errBuffer = DynamicBuffer("0x" + bytesToHex(self.cData))
                errBuffer.pos = 8
                if errBuffer.readInt() == 1:
                    self.errorIndex = errBuffer.readShort()
